/*
	mozphone.js
	Author: Jean-Denis Girard <jd.girard (at) sysnux.pf>
	License: Mozilla Public License or GPL

	mozphone.onLoad:
		-> mp_init: define gui objects
			-> ast_init: start network_client, load prefs, setup call objects, socket comms
*/

try {	// enablePrivilege is required if not running chrome'd
		// (other tweaks might apply, check out public.mozdev.jslib)
				netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
				include (jslib_socket);
				include (jslib_routines);
				include (jslib_file);
				include (jslib_fileutils);
				include (jslib_dir);
				include (jslib_window);
				include (jslib_dirutils);
} catch( e ) { alert( e ); }

// Localized strings bundle, accessible via strings.getString(`foo') or 
// strings.getFormattedString("fileInfo", [fileName, fileSize])
var strings = document.getElementById( "mp_strings" );

// Array of call objects; index is line number
var calls = new Array();

// proxiax to communicate with IAX network_client
var proxiax = new Object(); 

// GUI
var gui = new Object();

// Preferences object
var prefs = new Object();

function debug( s ) {
	if(  gui.debug && gui.debug.log ) gui.debug.log( s );
}

// Save preferences to file
function prefs_save() {
	var f = new File( prefs.pref_file );
/* XXX	if( f.exists() ) { 
	f.copy( home_dir + '.mozphone/prefs.OLD' );
		f.remove();
		} */
	if( f.open("w", 0600) != null ) {
		f.write( 'ast_host='     + prefs.ast_host + '\n');
		f.write( 'iax_port='     + prefs.iax_port + '\n');
		f.write( 'iax_peer='     + prefs.iax_peer + '\n');
		f.write( 'iax_secret='   + prefs.iax_secret + '\n');
		f.write( 'man_port='     + prefs.man_port + '\n');
		f.write( 'man_username=' + prefs.man_username + '\n');
		f.write( 'man_secret='   + prefs.man_secret + '\n');
		f.write( 'browser_path=' + prefs.browser_path + '\n' );
		f.write( 'concurrency='  + prefs.concurrency + '\n' );
		f.write( 'AGC='          + prefs.AGC + '\n' );
		f.write( 'EC='           + prefs.EC + '\n' );
		f.write( 'DN='           + prefs.DN + '\n' );
		f.write( 'silence='      + prefs.silence + '\n' );
		f.write( 'debug='        + prefs.debug + '\n' );
		f.write( 'gmanage='      + prefs.gmanage + '\n' );
		f.write( 'callconf='     + prefs.callconf + '\n' );
		f.write( 'autohide='     + prefs.autohide + '\n' );
		f.write( 'queues='       + prefs.queues + '\n' );
		f.write( 'fop='          + prefs.fop + '\n' );
		f.write( 'input='        + prefs.input + '\n' );
		f.write( 'output='       + prefs.output + '\n' );
		f.write( 'ring='         + prefs.ring + '\n' );
		f.write( 'ilbc='         + prefs.ilbc + '\n' );
		f.write( 'speex='        + prefs.speex + '\n' );
		f.write( 'gsm='          + prefs.gsm + '\n' );
		f.write( 'ulaw='         + prefs.ulaw + '\n' );
		f.write( 'alaw='         + prefs.alaw + '\n' );
		f.write( 'pref_codec='   + prefs.pref_codec + '\n' );
		var el = document.getElementById('dialpad').getAttribute('collapsed');
		f.write( 'dialpad=' );
		f.write( (el=='true') ? 'true':'false' );
		f.write( '\n' );
		el = document.getElementById('levels').getAttribute('collapsed');
		f.write( 'levels=' );
		f.write( (el=='true') ? 'true':'false' );
		f.write( '\n' );
		for( var i=0; i<gui.call_list.childNodes.length; i++ ) {
			f.write( 'history=' + gui.call_list.childNodes[i].label + ' ');
			f.write( gui.call_list.childNodes[i].getAttribute('tooltiptext') + '\n');
		}
		f.close();
	} else 
		alert( strings.getString('save_error') );
}

function new_prefs() {
	// Apply changes now
	// XXX what about network changes ???
	child_windows();
	filters();
	codecs();
	devices();
	return true;
}

function child_windows() {
	debug_window();
	gmanage_window();
	queues_window();
	fop_window();
	if( prefs.dialpad == 'true' ) {
		collapse('dialpad');
		document.getElementById( 'menu.dialpad' ).setAttribute( 'checked', 'false' );
	}
	if( prefs.levels == 'true' ) {
		collapse('levels');
		document.getElementById( 'menu.levels' ).setAttribute( 'checked', 'false' );
	}
}

function debug_window() {
	// debug window
	if( prefs.debug == 'true') {
		if( ! gui.debug ) { // Create window
			gui.debug = window.open('chrome://mozphone/content/debug.xul', 
				'debug_main', 'chrome,resizable,scrollbars');
		}
	} else {
		if( gui.debug ) { // Delete window
			gui.debug.close();
			gui.debug = 0;
		}
	}
}

function gmanage_window() {
	// semi-graphical manager window
	if( prefs.gmanage == 'true' ) {
		if( ! gui.man_logged_in ) {
			alert( strings.getString('man_not_logged_in') );
			if( gui.gmanage ) { // Delete window
				gui.gmanage.close();
				gui.gmanage = 0;
			}
		} else if( ! gui.gmanage ) { // Create window
			gui.gmanage = window.open('chrome://mozphone/content/gmanage.xul', 
				'gmanage_main', 'chrome,resizable,scrollbars');
		}
	} else {
		if( gui.gmanage ) { // Delete window
			gui.gmanage.close();
			gui.gmanage = 0;
		}
	}
}

function queues_window() {
	// Queues window
	if( prefs.queues == 'true' ) {
		if( ! gui.man_logged_in ) {
			alert( strings.getString('man_not_logged_in') );
			if( gui.queues ) { // Delete window
				gui.queues.close();
				gui.queues = 0;
			}
		} else if( ! gui.queues ) { // Create window
			gui.queues = window.open('chrome://mozphone/content/queues.xul', 
				'queues_main', 'chrome,resizable,scrollbars');
		}
	} else {
		if( gui.queues ) { // Delete window
			gui.queues.close();
			gui.queues = 0;
		}
	}
}

function fop_window() {
	// Flash Operator Panel
	if( prefs.fop != '' ) {
		if( ! gui.fop ) { // Create window
			gui.fop = window.open( prefs.fop, 'fop_main', 'resizable');
		}
	} else {
		if( gui.fop ) { // Delete window
			gui.fop.close();
			gui.fop = 0;
		}
	}
}

function mp_init() { // MozPhone initialization (called by onLoad event)
	
	// Splash screen !
	mp_splash();

	// Populate gui object
	gui.call_adr   = document.getElementById( "call_adr" );
	gui.bcall      = document.getElementById( "bcall" );
	gui.in_meter   = document.getElementById( "in_meter" );
	gui.out_meter  = document.getElementById( "out_meter" );
	gui.registration = document.getElementById( "registration" );
	gui.call_adr.focus();
	gui.call_list = document.getElementById( "call_list" );
	gui.GUID = '4de2cc0c-a9e4-4853-871c-7cda72582e13'; // MozPhone GUID
	gui.man_logged_in = false;

	// "Quit" no longer exported in Firefox 1.0 ?
	if( window.quit!=undefined && typeof(window.quit)=='function' ) {
		// Append quit menuitem
		menu = document.getElementById( "menu-popup" );
		// <menuitem label="&menu.quit;" oncommand="mp_quit();quit();" />
		var item = document.createElement( 'menuitem' );
		item.setAttribute( 'label', strings.getString('quit') );
		item.setAttribute( 'oncommand', 'mp_close();quit();' );
		menu.appendChild(item);
	}

	//  Then load preferences
	prefs_load();

	// Then instantiate call objects
	if( prefs.concurrency ) {
		if( prefs.concurrency > Call.MAXCALLS ) 
			prefs.concurrency = Call.MAXCALLS;
		else 
			Call.MAXCALLS = prefs.concurrency;
	} else 
			prefs.concurrency = Call.MAXCALLS;
	for( var i=0; i<Call.MAXCALLS+1; i++ )
		calls[i] = new Call(i, 'call_strings');

	// Then start asterisk communications through network_client
	ast_init();
}

function mp_splash() {
	const url = "chrome://mozphone/content/splash.xul";
	var win = new CommonWindow(url, 350, 300);
	win.position = JS_MIDDLE_CENTER;
	win.openSplash();
}

function debug_close() { // Called on unload event from debug window
	gui.debug = 0;
}

function gmanage_close() { // Called on unload event from gmanager window
	debug( 'gmanage_close()\n' );
	gui.gmanage = 0;
}

function queues_close() { // Called on unload event from queues window
	debug( 'queues_close()\n' );
	gui.queues = 0;
}

function fop_close() { // Called on unload event from fop window
	debug( 'fop_close()\n' );
	gui.fop = 0;
}

/*
# Gnophone type preferences (~/.gnophone/telco.cfg)
mode=2
iaxserver=asterisk.sysnux.pf
iaxcontext=gnophone
iaxusername=jdg
iaxpassword=jdg
iaxpeer=jdg
iaxsecret=jdg
iaxprefix=
iaxport=5036

# GastMan type preferences (~/.gastman/asterisk.sysnux.pf.extens)
:Modem[i4l]/ttyI0:programmer:333:236:OPT2
0:Modem[i4l]/ttyI0:programmer:324:86:OPT1
12:SIP/sip2:programmer:86:186:Laurent
11:SIP/sip1:programmer:88:53:Jean-Denis
*/
// Show preferences dialog
function prefs_dlg() {
	ret = new Object(); // Dialog returning value
	ret.res = 0;
	window.openDialog('chrome://mozphone/content/preferences.xul',	'prefs_main', 'chrome,resizable,scrollbars,dialog=yes,close,modal=yes', prefs, ret);
	if(ret.res) {
		prefs_save();
		if(ret.res==2) {
			if( confirm( strings.getString('restart') )) {
				mp_close();
				if( window.quit!=undefined && typeof(window.quit)=='function' )
					quit();
			}
		}
		new_prefs();
	}
}

function prefs_load() {
	var du = new DirUtils();
	var fu = new FileUtils();

	// Search for preferences in mozphone directory (~/.mozphone)
	// Should we use Mozilla's preferences infrastructure ?
	var d = new Dir( du.getHomeDir() );
	d.append( '.mozphone' );
	if( ! d.exists() ) {
		d.create();
	}
	prefs.pref_file = fu.append( d.path, 'prefs' );;
	var f = new File( prefs.pref_file );
	if( ! f.exists() ) { 
		prefs_dlg();
	}
	f.open();
	var a = new Array();
	// Parse preferences
	prefs.fop = '';
	prefs.devices = new Array();
	while( !f.EOF ) {
		var l = f.readline();
		a = l.split('=');
		switch( a[0] ) {
			case 'ast_host':
				prefs.ast_host = a[1];
				break;
			case 'iax_port':
				prefs.iax_port = a[1];
				break;
			case 'iax_peer':
				prefs.iax_peer = a[1];
				break;
			case 'iax_secret':
				prefs.iax_secret = a[1];
				break;
			case 'man_port':
				prefs.man_port = a[1];
				break;
			case 'man_username':
				prefs.man_username = a[1];
				break;
			case 'man_secret':
				prefs.man_secret = a[1];
				break;
			case 'browser_path':
				prefs.browser_path = a[1];
				break;
			case 'concurrency':
				prefs.concurrency = a[1];
				break;
			case 'AGC':
				prefs.AGC = a[1];
				break;
			case 'EC':
				prefs.EC = a[1];
				break;
			case 'DN':
				prefs.DN = a[1];
				break;
			case 'silence':
				prefs.silence = a[1];
				break;
			case 'debug':
				prefs.debug = a[1];
				debug_window();
				break;
			case 'gmanage':
				prefs.gmanage = a[1];
				break;
			case 'callconf':
				prefs.callconf = a[1];
				break;
			case 'autohide':
				prefs.autohide = a[1];
				break;
			case 'queues':
				prefs.queues = a[1];
				break;
			case 'fop':
				prefs.fop = a[1];
				break;
			case 'input':
				prefs.input = a[1];
				break;
			case 'output':
				prefs.output = a[1];
				break;
			case 'ring':
				prefs.ring = a[1];
				break;
			case 'ilbc':
				prefs.ilbc = a[1];
				break;
			case 'speex':
				prefs.speex = a[1];
				break;
			case 'gsm':
				prefs.gsm = a[1];
				break;
			case 'ulaw':
				prefs.ulaw = a[1];
				break;
			case 'alaw':
				prefs.alaw = a[1];
				break;
			case 'pref_codec':
				prefs.pref_codec = a[1];
				break;
			case 'history':
				var x = a[1].split(' ');
				if( x.length < 3 ) {
					x[2] = '1900/01/01';
					x[3] = '00:00:00';
				}
				//append_call_list( x[0], x[1], x[2], x[3] );
				var item = document.createElement( 'menuitem' );
				item.setAttribute( 'label', x[0] + ' ' + x[1] );
				item.setAttribute( 'value', x[1] + ' ' + x[2] + ' ' + x[3] );
				item.setAttribute( 'tooltiptext', x[2] + ' ' + x[3] );
				gui.call_list.appendChild(item);
				break;
			case 'dialpad':
				prefs.dialpad = a[1];
				break;
			case 'levels':
				prefs.levels = a[1];
				break;
			default:
				debug("Unknown preference: <" + l + ">\n" );
				break;
		}
	}
}

function ast_init() {
	// Search for proxiax host / port in environment
	proxiax.host = '127.0.0.1'; // Default values
	proxiax.port = 9998;
	var env = Components.classes["@mozilla.org/process/environment;1"].
		getService(Components.interfaces.nsIEnvironment);
	if( env.exists('PROXIAX_HOST') )	 {
		proxiax.host = env.get('PROXIAX_HOST');
		if( env.exists('PROXIAX_PORT') )
			proxiax.port = env.get('PROXIAX_PORT');
	}

	if( proxiax.host == '127.0.0.1' ) {
		// Look for network_client executable and spawn it 
		// only if on same host !
		var os = navigator.platform;
		var name = 'network_client';
		var du = new DirUtils();
		var fu = new FileUtils();
		var comp = new Dir( du.getComponentsDir() );
		var network_client = fu.append( comp.path, name );
		var rv;
		if( /win/i.test(os) ) { // Running on Windows
			// Hack to hide DOS console by launching network_client.exe 
			// through CreateProcess in nowin.exe
			// Is there a better way ?
			var no_win = fu.append( comp.path, 'nowin.exe' );
			network_client = network_client + '.exe'; 
			if( ! fu.exists(network_client) ) {
				alert( strings.getString('nc_error') );
				mp_close();
			}
			if( fu.exists(no_win) ) {
				rv = fu.spawn( no_win, [ network_client ] );
			} else {
				rv = fu.spawn( network_client );
			}
		// } else if( /mac/i.test(os) )  XXX
		// XXX What about Running on Mac ?
		} else { // Running on Linux
			if( ! fu.exists(network_client) ) {
				alert( strings.getString('nc_error') );
				mp_close();
			}
			rv = fu.spawn( network_client );
		}
	}

	// Now try to connect to network_client
	proxiax.retry = 0;
	proxiax.data = '';
	proxiax.sock = new Socket();
	proxiax_connect();
}

function close_socket( sock ) {
 	sock.close();
	debug( "Socket was closed\n" );
}

function open_socket( sock, host, port )  {
	if( sock.isConnected ) 
		return;
	sock.open( host, port, true ); // Open returns nothing !
}

function proxiax_connect() {
	// Connect to network_client
	open_socket( proxiax.sock, proxiax.host, proxiax.port );
	debug( "proxiax socket " +  proxiax.host + ":" + proxiax.port + " openned... " );
	if( ! proxiax.sock.isAlive() ) {
		debug( 'but not alive, retry ' + proxiax.retry + ' !\n');
		if( ! /win/i.test(navigator.platform) ) // Running on Windows
			close_socket( proxiax.sock );
		if( proxiax.retry == 20 ) { // No connection, quit now !
			alert( strings.getString('proxiax_conn') );
			mp_close();
		}
		proxiax.retry++;
		setTimeout( "proxiax_connect()", 200 );
		return;
	}
	debug( 'and alive on retry ' + proxiax.retry + ' !\n');
	proxiax.retry = 0;
	proxiax_synchro();
}

function proxiax_synchro() {
	// Synchronize with network_client
	var data = receive_socket( proxiax.sock );
	if( data ) {
		proxiax.data += data;
	}
	var iaxcver = proxiax.data.match( /Hello\s(.*)/ ); 
	if( ! iaxcver ) {
		debug( 'proxiax waiting for synchro, retry ' + proxiax.retry + '... <' 
				+ proxiax.data + '>\n' );
		if( proxiax.retry == 20 ) { // No synchro, quit now !
			alert( strings.getString('proxiax_sync') );
			mp_close();
		}
		proxiax.retry++;
		setTimeout( "proxiax_synchro()", 100 );
		return;
	}
	proxiax.iaxcver = iaxcver[1];
	debug( "proxiax synchro OK, iaxclient_version " + proxiax.iaxcver + "\n" );

	// ProxIAX should be OK now, start receiving "thread"
	proxiax.data = '';
	proxiax_read();

	// Register as IAX peer with asterisk server
	proxiax_send( 'iax_register,' + prefs.iax_peer + ',' + 
		prefs.iax_secret + ',' + prefs.ast_host );
	gui.registration.label = prefs.ast_host;
	codecs();

	// Connect to asterisk manager (same host)
	if( prefs.man_username!="" && prefs.man_secret!="" )
		proxiax_send( "man_login," + prefs.man_username + ',' + prefs.man_secret );

	// Get / Set list of audio devices: list of devices will be obtained asynchronously
	// so we can't rely on it here, just use blindly the prefs
	proxiax_send( 'get_devices' );
	proxiax_send( 'set_devices,' + prefs.input  + ',' + prefs.output  + ',' + prefs.ring );

	// Apply filters
	filters();

	// Display other windows (delay needed for manager login)
	setTimeout( 'child_windows()', 1000 );

	// Did we receive arguments ?
	if( window.arguments && window.arguments[0] ) {
		gui.call_adr.value = window.arguments[0].url;
      call();
	}
}

function proxiax_read() {
	var data = receive_socket( proxiax.sock );
	if( data ) {
		data = proxiax.data + data;
		var index = data.lastIndexOf( '\r\n' );
		if( index != data.length-2 ) { // Message is not complete
			proxiax.data = data.slice( index+2 );
		}
		var strings = new Array;
		strings = data.split( '\r\n' );
		for( var i=0; i<strings.length-1 ; i++ ) {
			proxiax_decode( strings[i] );
		}
	}

	// Update call(s) duration(s)
	for( var i=0; i<Call.MAXCALLS; i++ ) {
		calls[i].dur_timer();
	}
	proxiax_timer = setTimeout( "proxiax_read()", 250 ); // XXX
}

function open_browser( uri ) {	
	if( ! prefs.browser_path ) {
		debug( 'state: loading url ' + uri + ' in mozilla Window...\n' );
		// Native Browser Window
		browser = open( uri, 'CRM' );
	} else {
		// External browser
		debug( 'BROWSER_PATH = ' + prefs.browser_path + '\n' );
		var fu = new FileUtils();
		if( ! fu.exists(prefs.browser_path) ) {
			debug('ERREUR ' + prefs.browser_path + '\n');
		} else {
			debug('RUN ' + prefs.browser_path + ' ' + uri + '\n' );
			var rv = fu.spawn( prefs.browser_path, [uri] );
			debug('RUNNING (rv = ' + rv + ') ?\n' );
		}
	}
}

function proxiax_decode( s ) {
	a = new Array;
	a = s.split( ',' );
	switch( a[0] ) {
		case 'registration':
			// Result of IAXclient registration to * server
			switch( a[1] ) {
				case 'ack': // IAX event registration ack
					gui.registration.setAttribute('style','color: green');
					gui.registration.setAttribute('tooltiptext', 
							strings.getString('registration_ack'));
					break;
				case 'rej': // IAX event registration reject
				case 'failed': // IAXclient error
					gui.registration.setAttribute('style','color: red');
					alert( strings.getString('registration_error') );
					gui.registration.setAttribute('tooltiptext', 
							strings.getString('registration_error'));
					break;
				case 'tmo': // IAX event registration timeout
					gui.registration.setAttribute('style','color: yellow');
					gui.registration.setAttribute('tooltiptext', 
							strings.getString('registration_tmo'));
					break;
				case 'id': // IAXclient registration ID
					proxiax.regid = a[2];
					break;
				default:
					debug( 'registration ??? ' + s + '\n' );
					break;
			}
			break;
		case 'device':
			// New device, parameters: number,name,capacities
			dev = new Object();
			dev.name = a[2];
			dev.caps = a[3];
			prefs.devices[a[1]] = dev;
			break;
		case 'levels':
			// Niveaux -60..0 => 0..100
			gui.in_meter.value  = (60+parseInt(a[1]))*100/60;
			gui.out_meter.value = (60+parseInt(a[2]))*100/60;
			break;
		case 'state':
			/* State event: state,callNo,cause,parameters
				cause,parameters is one of: 
					not_active
					ring_out
					active,remote
					ring_in,remote
					url,url
			*/
			var call_num = parseInt( a[1] );
			var cause = a[2];
			debug( 'state: ' + cause + ' (' + call_num + ')\n' );
			switch( cause ) {
				case 'ring_out':
					calls[call_num].ringout();
					break;
				case 'active':
					calls[call_num].activated( a[3] );
					break;
				case 'selected':
					calls[call_num].selected();
					break;
				case 'ring_in': // Ringing in !
					calls[call_num].ringing( a[3] );
					break;
				case  'not_active': // Disconnected !
					calls[call_num].disconnected();
					break;
				case 'url': // Display received URL in preferred browser
					uri = a[3];
					if( uri ) open_browser( uri );
					else debug( 'state: received url but NO URI !\n' );
					break;
				default: // Unknown state !
					debug( 'ERROR unknown state=' + cause +  ' ' + call_num + '\n' );
					break;
			}
			break;
		case 'text':
			/* Not interested in text messages
			var call_num = parseInt( a[1] );
			var type = a[2];
			var msg = a[3];
			if( msg.indexOf( 'Dropping' ) == -1 ) {// 
				debug( 'text:' + msg + ' / ' + call_num + '\n' );
			}
			*/
			break;
		case 'manager':
			// Manager event: manager,type,data
			var type = a[1];
			var data = a[2];
			if( type == 'login' ) {
				if( data == 'success' ) {
					gui.man_logged_in = true;
				} else {
					alert( strings.getString('man_login_failed') );
				}
			} else 
				man_decode( type, data );
			break;
		default:
			debug( 'PROXIAX ??? ' + s + '\n' );
			break;
	}
}

function man_decode( type, data ) {
	if( ! gui.man_logged_in ) 
		return;
	if( gui.gmanage && gui.gmanage.decode ) {
		gui.gmanage.decode( type, data );
	}
	if( gui.queues && gui.queues.decode ) {
		gui.queues.decode( type, data );
	}
}

function proxiax_send( msg ) {
	proxiax.sock.write( msg + '\r\n' );
}

function receive_socket( sock ) {
	bytes = sock.available();
	if( ! bytes ) return;
	data = sock.read( bytes );
	return data
}

// Manage IAXclient filters
function filters() {
	var x = '';
	if( prefs.AGC == 'true' ) x += 'A';
	if( prefs.EC  == 'true' ) x += 'E';
	if( prefs.DN  == 'true' ) x += 'D';
	proxiax_send( 'filters,' + x );
	proxiax_send( 'silence,' + prefs.silence );
	debug( 'filters,' + x + ' silence,' + prefs.silence + '\n' );
	return true;
}

// IAX codecs
function codecs() {
	var allow = '';
	allow += (prefs.ilbc=='true')  ? 'ilbc ' :'';
	allow += (prefs.speex=='true') ? 'speex ':'';
	allow += (prefs.gsm=='true')   ? 'gsm '  :'';
	allow += (prefs.alaw=='true')  ? 'alaw ' :'';
	allow += (prefs.ulaw=='true')  ? 'ulaw ' :'';
	var cmd = 'set_formats,' + prefs.pref_codec + ',' + allow;
	proxiax_send( cmd );
	debug( cmd + '\n' );
}

// IAX devices
function devices() {
	var cmd = 'set_devices,' + prefs.input + ',' + prefs.output + ',' + prefs.ring;
	proxiax_send( cmd );
	debug( cmd + '\n' );
}

function digit( c ) {
	// If no call is selected, digit is added to call number
	// else digit is sent in selected call
	if( Call.selected == -1 )
		gui.call_adr.value += c;
	else
		proxiax_send( c );
	return true;
}

function append_call_list( s, num ) {
	// Timestamp
	var d = new Date();
	var yy = d.getFullYear();
	var mo = d.getMonth();
	var da = d.getDate();
	var ho = d.getHours();
	var mi = d.getMinutes();
	var se = d.getSeconds();
	var date = yy + ((mo<10) ? '/0':'/') + mo + ((da<10) ? '/0':'/') + da +
		((ho<10) ? ' 0':' ') + ho + ((mi<10) ? ':0':':') + mi + ((se<10) ? ':0':':') + se;

	// Check call is not stored aleady
	list = gui.call_list;
	for( var i=0; i<list.childNodes.length; i++ ) {
		n = (list.childNodes[i].value).split(' ');
		if(  num == n[0] ) { // Update timestamp
			list.childNodes[i].setAttribute( 'value', num + ' ' + date);
			list.childNodes[i].setAttribute( 'tooltiptext', date);
			return;
		}
	}
	// Add incoming call to history
	var item = document.createElement( 'menuitem' );
	item.setAttribute( 'label', s + ' ' + num );
	item.setAttribute( 'value', num + ' ' + date);
	item.setAttribute( 'tooltiptext', date);
	list.appendChild(item);
}

function call() {
	var num = gui.call_adr.value;
	// History adds "< " or "> " to number
	if( (num.charAt(0) == '<') ||  (num.charAt(0) == '>') )
		num = num.substring(2);
	if( num.length < 1 ) {
		alert( strings.getString('no_num') );
		gui.call_adr.focus();
		return false;
	}
	if( prefs.callconf=='true' && !confirm( strings.getFormattedString('confirm_call', [num]) ) )
		return false;
	proxiax_send( 'call,' + prefs.iax_peer + ':' + prefs.iax_secret + '@' + 
			prefs.ast_host + '/' + num );
	gui.call_adr.value = '';
	append_call_list( '>', num );
	return true;
}

// MozPhone close
function mp_close() {
	if( proxiax.sock.isAlive() ) {
		proxiax_send( 'quit' );
		// close_socket( proxiax );
	}
	if( gui.debug   ) gui.debug.close();
	if( gui.gmanage ) gui.gmanage.close();
	if( gui.queues  ) gui.queues.close();
	if( gui.fop     ) gui.fop.close();
	if( gui.help    ) gui.help.close();
	prefs_save();
	close();
}

function mp_about() {
	window.openDialog('chrome://mozphone/content/about.xul', 'about_main', 
			'chrome,resizable,scrollbars,dialog=yes,close,modal=yes' );
}

function mp_help() {
	gui.help = window.open( 'chrome://mozphone/content/help/index.html', 
			'help', 'resizable');
}

function collapse(element) {
	// Toggle show / hide GUI element
	var offset=10;
	if( element == 'dialpad' )
		offset=0; // Dialpad is not a complete box, no offset
	el = document.getElementById(element);
	if( el.getAttribute('collapsed') == 'true' ) {
		el.setAttribute('collapsed', false );
		self.resizeBy( 0, offset + el.boxObject.height );
	} else {
		self.resizeBy( 0, -(offset + el.boxObject.height) );
		el.setAttribute('collapsed', true );
	}
}


/*************** DEAD CODE: attempt to install using Extension Manager
function install_exe() {
		// Binary copy network_client from chrome:JARchive to file system 
		var name = 'network_client';
		if( /win/i.test(os) ) name += '.exe';
		// else if( /mac/i.test(os)) exe = XXX

		var ios = jslibGetService('@mozilla.org/network/io-service;1','nsIIOService');
		var uri = ios.newURI('chrome://mozphone/content/' + name, null, null);
		var channel = ios.newChannelFromURI(uri);
		var is = jslibCreateInstance('@mozilla.org/binaryinputstream;1', 'nsIBinaryInputStream');
		is.setInputStream(channel.open());

		var exe = new File( fu.append( home.path, name) );
		if( exe.exists() && exe.size != is.available() ) {
			// Remove old file
			exe.remove();
alert( 'old ' + name + ' removed !' );
		}
		if( ! exe.exists()  ) {
			exe.mFileInst.create( JS_FILE_FILE_TYPE, 0755 ); // Workaround JSlib bug with file permissions
			exe.open( 'wb', 0755 );
			exe.write( is.readByteArray(is.available() ) );
			exe.close();
alert( name + ' created !' );
		}
}
*********************/
